"use strict";
/*
    Migrate.js - OneTable Migrations
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Migrate = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const semver_1 = __importDefault(require("semver"));
const dynamodb_onetable_1 = require("dynamodb-onetable");
const MigrationFields = {
    pk: { type: String, value: '_migrations:' },
    sk: { type: String, value: '_migrations:${version}' },
    description: { type: String, required: true },
    date: { type: Date, required: true },
    path: { type: String, required: true },
    version: { type: String, required: true },
};
class Migrate {
    /*
        params: {
            migrations: [in-memory-migrations]
            dir: 'migrations-directory'
        }
     */
    constructor(db, params = {}) {
        this.params = params;
        this.db = db;
        this.migrations = params.migrations;
        this.dir = path_1.default.resolve(params.dir || '.');
        this.Migration = new dynamodb_onetable_1.Model(db, '_Migration', { fields: MigrationFields }, { timestamps: false });
    }
    /* public */
    findPastMigrations() {
        return __awaiter(this, void 0, void 0, function* () {
            let pastMigrations = yield this.Migration.find();
            this.sortMigrations(pastMigrations);
            return pastMigrations;
        });
    }
    /* public */
    apply(direction, version) {
        return __awaiter(this, void 0, void 0, function* () {
            let migration;
            if (direction == 0) {
                yield this.Migration.remove({}, { many: true });
                //  Create prior migration items
                let versions = yield this.getVersions();
                for (let v of versions) {
                    migration = yield this.loadMigration(v);
                    let params = {
                        version: v,
                        date: new Date(),
                        path: migration.path,
                        description: migration.description,
                    };
                    if (v == version) {
                        yield migration.up(this.db, this);
                    }
                    yield this.Migration.create(params);
                }
            }
            else {
                migration = yield this.loadMigration(version);
                if (direction < 0) {
                    yield migration.down(this.db, this);
                    yield this.Migration.remove({ version: migration.version });
                }
                else {
                    yield migration.up(this.db, this);
                    let params = {
                        version,
                        date: new Date(),
                        path: migration.path,
                        description: migration.description,
                    };
                    yield this.Migration.create(params);
                }
            }
            return migration;
        });
    }
    /* public */
    getCurrentVersion() {
        return __awaiter(this, void 0, void 0, function* () {
            let pastMigrations = yield this.findPastMigrations();
            if (pastMigrations.length == 0) {
                return '0.0.0';
            }
            return pastMigrations[pastMigrations.length - 1].version;
        });
    }
    /*
        Return outstanding versions in semver sorted order up to the specified limit
     */
    /* public */
    getOutstandingVersions(limit = Number.MAX_SAFE_INTEGER) {
        return __awaiter(this, void 0, void 0, function* () {
            let current = yield this.getCurrentVersion();
            let pastMigrations = yield this.findPastMigrations();
            let versions = yield this.getVersions();
            versions = versions.filter(version => {
                return semver_1.default.compare(version, current) > 0;
            }).sort(semver_1.default.compare);
            versions = versions.filter(v => pastMigrations.find(m => m.version == v) == null);
            return versions.slice(0, limit);
        });
    }
    /* private */
    getVersions() {
        return __awaiter(this, void 0, void 0, function* () {
            let versions;
            if (this.migrations) {
                versions = this.migrations.map(m => m.version);
            }
            else {
                versions = fs_1.default.readdirSync(this.dir).map(file => file.replace(/\.[^/.]+$/, ''));
            }
            versions = versions.filter(version => { return semver_1.default.valid(version); }).sort(semver_1.default.compare);
            return versions;
        });
    }
    /* private */
    loadMigration(version) {
        return __awaiter(this, void 0, void 0, function* () {
            let path, task;
            if (this.migrations) {
                task = this.migrations.find(m => m.version == version);
            }
            else {
                path = `${this.dir}/${version}.js`;
                task = (yield Promise.resolve().then(() => __importStar(require(path)))).default;
            }
            if (!task) {
                throw new Error(`Cannot find migration for version ${version}`);
            }
            return {
                version,
                description: task.description,
                path: path || 'memory',
                up: task.up,
                down: task.down,
            };
        });
    }
    /* private */
    sortMigrations(array) {
        array.sort((a, b) => {
            let cmp = semver_1.default.compare(a.version, b.version);
            if (cmp < 0) {
                return cmp;
            }
            else if (cmp > 0) {
                return cmp;
            }
            else if (a.order < b.order) {
                return -1;
            }
            else if (a.order > b.order) {
                return 1;
            }
            else {
                return 0;
            }
        });
    }
}
exports.Migrate = Migrate;
